locals {
  replication_group_id = "${var.service_name}-cacherep"
  #The AUTH token modification is only supported for redis engine version 5.0.5 and above.
  cluster_mode_pg = {
    "5.0.6" = "default.redis5.0.cluster.on"
  }

}

resource "aws_elasticache_replication_group" "cache_rep" {
  # Required
  replication_group_id          = var.service_name
  replication_group_description = "elasticache redis replication group"
  node_type                     = var.redis_node_type

  # Optional
  engine                     = "redis"
  automatic_failover_enabled = true
  multi_az_enabled           = var.multi_availability_zone

  # Lock it down - note, this causes some clients to have difficulty connecting due to TLS and cert/ca verification
  at_rest_encryption_enabled = true
  transit_encryption_enabled = true
  auth_token                 = var.redis_password
  subnet_group_name          = aws_elasticache_subnet_group.subnet_group.name
  security_group_ids         = var.security_group_ids
  engine_version             = var.redis_engine_version
  parameter_group_name       = lookup(local.cluster_mode_pg, var.redis_engine_version)
  port                       = var.redis_port

  maintenance_window       = var.redis_maintenance_window
  snapshot_window          = var.redis_snapshot_window
  snapshot_retention_limit = var.redis_snapshot_retention_limit


  cluster_mode {
    replicas_per_node_group = var.replica_count
    num_node_groups         = var.shard_count
  }

  lifecycle {
    ignore_changes = [
      auth_token,
    ]
  }

  tags = var.tags
}

resource "aws_elasticache_subnet_group" "subnet_group" {
  name        = "${var.service_name}-ec-subnet"
  description = "elasticache subnet for ${var.service_name}"
  subnet_ids  = var.subnet_ids
  tags        = var.tags
}
